let bailout = 20 ;;

(* Algorithme de generation de l'ensemble de Mandelbrot, technique classique *)

let mandelbrot x y dim = 
	let res = float_of_int dim in
	let a = (2.6 *. (float_of_int x) /. res) -. 2. in
	let b = (2.6 *. (float_of_int y) /. res) -. 1.3 in
	let c = ref 0. in
	let d = ref 0. in
	let cp = ref 0. in
	let i = ref 0 in
	
	while (!i < 200) && ((!c)**2. +. (!d)**2. < 4.) do
		cp := !c**2. -. !d**2. +. a;
		d := 2. *. !c *. !d +. b;
		c := !cp;
		incr i;
	done;
	
	if !i = 200 then Graphics.rgb 0 0 0
	else Graphics.rgb (!i+15) !i (!i+55)

	
	
(* Technique de generation de l'ensemble de Julia, methode classique *)

(* les valeurs de c possible:
-1.26267 -0.04063i
0.285 + 0.0122i
-1.417022285618 + 0.0099534i
-0.038088 + 0.9754633i
-0.156844471694257101941 -0.649707745759247905171i
*)

let u = -1.26267;;
let v = -0.04063;;

let julia x y dim = 
	let res = float_of_int dim in
	let a = ref ((3. *. (float_of_int x) /. res) -. 1.5) in
	let b = ref ((3. *. (float_of_int y) /. res) -. 1.5) in
	let ap = ref !a in
	let i = ref 0 in
	
	while (!i < 400) && ((!a)**2. +. (!b)**2. < 4.) do
		ap := !a**2. -. !b**2. +. u;
		b := 2. *. !a *. !b +. v;
		a := !ap;
		incr i;
	done;
	
	if !i = 400 then Graphics.rgb 0 0 0
	else Graphics.rgb (if !i<50 then !i+100 else !i/4+138)
					  (if !i<5 then 0 else if !i<25 then (!i-5)*12 else if !i<100 then 265- !i*2 else if !i<235 then 90 else !i-145)
					  (if !i<10 then !i*12 else if !i<40 then !i*4+80 else if !i<120 then 320- !i*2 else if !i <200 then 80 else !i/2-20)

	(*
	let j = cos((float_of_int !i) /. 4.12) in
	let l = sin((float_of_int !i) /. 100.) in
	let k = sin((float_of_int !i) /. 32.81) in
	if !i = 400 then Graphics.rgb 0 0 0
	else Graphics.rgb (int_of_float( 255. *. abs(j)))
					  (int_of_float( 100. *. abs(l)))
					  (int_of_float( 155. *. abs(k) +. 50.));;
	*)
	
	
(* Algortihme de generation de l'ensemble de Mandelbrot : methode Buddhabrot *)



let mandelbrot x y = 

	let c = ref 0. in
	let d = ref 0. in
	let temp = ref 0. in
	let i = ref 0 in
	
	while (!i < bailout) && ((!c)**2. +. (!d)**2. < 4.) do
		temp := !c**2. -. !d**2. +. x;
		d := 2. *. !c *. !d +. y;
		c := !temp;
		incr i;
	done;
	
	!i
	
	;;

let budhabrot max_iter = 

	let size = float_of_int (Graphics.size_x()) in
	
	let conversionX x =
		 int_of_float (((x +. 2.) /. 2.6) *. size) in
	
	let conversionY x =
		 int_of_float (((x +. 1.3) /. 2.6) *. size) in
		 
	Random.self_init();
	
	let tab = Array.create_matrix (Graphics.size_x()) (Graphics.size_x()) 0. in
	
	for j = 0 to (max_iter - 1)  do
	
		let randx = ((Random.float 2.6) -. 2.) in
		let randy = ((Random.float 2.6) -. 1.3) in
		let color = ref 0 in
		
		if (mandelbrot randx randy) <> bailout then
			begin
			let c = ref 0. in
			let d = ref 0. in
			let cp = ref 0. in
			let i = ref 0 in
			let abs_c , abs_d = ref (conversionX randx) , ref (conversionY randy) in
	
			while (!i < 100) && !abs_c < Graphics.size_x() && !abs_d < Graphics.size_x() && !abs_c >= 0 && !abs_d >= 0 do
			
				tab.(!abs_c).(!abs_d) <- tab.(!abs_c).(!abs_d) +. 1. ;
				
				color := int_of_float (tab.(!abs_c).(!abs_d));
				
				if !color >= 255 then Graphics.set_color (Graphics.rgb 0 127 255)
				else Graphics.set_color (Graphics.rgb 0 (!color/2) !color);
				
				Graphics.plot (!abs_d) (Graphics.size_x() - !abs_c - 1);
				
				
				cp := !c**2. -. !d**2. +. randx;
				d := 2. *. !c *. !d +. randy;
				c := !cp;
				incr i;
				
				abs_c := conversionX !c;
				abs_d := conversionY !d;
				
			done;
			
			end
	
	done
	;;

let transformation tab = 

	let long = Array.length tab - 1 in
	let tab_image = Array.create_matrix (long + 1) (long + 1) 0 in
	
	let max_matrix tab = 
		let max_t = ref 0. in
		
		for i = 0 to long do
			for j = 0 to long do
				max_t := max !max_t tab.(i).(j)
			done
		done;
		!max_t in
			
	let max = max_matrix tab in
	let couleur = ref 0 in
	
	for i = 0 to long do
		for j = 0 to long do
			couleur := int_of_float (255. *. tab.(i).(j) /. max);
			tab_image.(j).(i) <- ( Graphics.rgb 0 (!couleur/2) (!couleur) );
		done;
	done;
	
	tab_image ;;
